# make importing these a bit less hassle
from flexget.utils.parsers.movie import MovieParser  # noqa pylint: disable=unused-import
from flexget.utils.parsers.parser import TitleParser  # noqa pylint: disable=unused-import
from flexget.utils.parsers.series import (  # noqa pylint: disable=unused-import
    ID_TYPES,
    SeriesParser,
)
